# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/aws.ipynb.

# %% auto 0
__all__ = ['AwsClient']

# %% ../nbs/aws.ipynb 3
import zipfile
import boto3
import tempfile
import shutil
import os

# %% ../nbs/aws.ipynb 4
class AwsClient:

    def __init__(self, aws_access_key_id: str, aws_secret_access_key: str, region_name: str):

        s3 = boto3.client(
            's3',
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
            region_name=region_name
        )

        self.s3 = s3

    def upload_to_s3(self, file_path: str, bucket_name: str, object_name: str):

        self.s3.upload_file(file_path, bucket_name, object_name)

    def extract_zip_with_correct_encoding(self, zip_file_path, extract_to_path, encoding='cp437'):
        with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
            for member in zip_ref.infolist():
                # ファイル名のエンコーディングを修正
                file_name = member.filename.encode('cp437').decode(encoding)
                target_path = os.path.join(extract_to_path, file_name)
                
                # ディレクトリの場合は作成
                if member.is_dir():
                    os.makedirs(target_path, exist_ok=True)
                else:
                    # ファイルを展開
                    with zip_ref.open(member, 'r') as source, open(target_path, 'wb') as target:
                        shutil.copyfileobj(source, target)

    def upload_to_s3_zip(self, zip_file_path: str, dst_path: str, bucket_name: str):
        # S3クライアントの作成
        s3_client = self.s3

        # ZIPファイルの展開

        # self.extract_zip_with_correct_encoding(zip_file_path, extract_to_path, encoding='cp437')

        with tempfile.TemporaryDirectory() as tmpdirname:
            # ZIPファイルの展開
            with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
                zip_ref.extractall(tmpdirname)

            # 展開したファイルをS3にアップロード
            for root, dirs, files in os.walk(tmpdirname):
                for file in files:
                    local_path = os.path.join(root, file)

                    s3_path = os.path.join(dst_path, os.path.relpath(local_path, tmpdirname)).replace("\\", "/")

                    # 実際のアップロード
                    s3_client.upload_file(local_path, bucket_name, s3_path)
